package org.zipcoder.store;

import java.util.*;

public class MyHashMap implements MyMap{
    // final field that doesn't change for any object
    private static final int BUCKET_SIZE = 15;

    // instance field that can only be set once. It is different for every Object
    private final List<Entry>[] entries;

    public MyHashMap(){
        entries = new List[BUCKET_SIZE];
        for(int i = 0; i < BUCKET_SIZE; i++) {
            entries[i] = new ArrayList<>();
        }
    }

    @Override
    public int size() {
        int totalEntries = 0;
        for(int i = 0; i < BUCKET_SIZE; i++) {
            totalEntries += entries[i].size();
        }
        return totalEntries;
    }

    @Override
    public boolean isEmptry() {
        return (size() == 0);
    }

    @Override
    public Cart get(User key) {
        int bucket = bucketIndex(key);
        Iterator<Entry> entryIterator = entries[bucket].iterator();
        Entry retrievedEntry;
        Cart retrievedCart = null;
        while (entryIterator.hasNext()){
            retrievedEntry = entryIterator.next();
            if (retrievedEntry.getKey().equals(key)){
                retrievedCart = retrievedEntry.getValue();
            }
        }
        return retrievedCart;
    }

    @Override
    public void put(User key, Cart value) {
        int bucket = bucketIndex(key);
        Entry newEntry = new Entry(key, value);
        entries[bucket].add(newEntry);
    }

    @Override
    public List<User> getKeys() {
        List<User> entriesUser = new ArrayList<>();
        for (int i = 0; i < BUCKET_SIZE; i++) {
            for (Entry e : entries[i]) {
                entriesUser.add(e.getKey());
            }
        }
        return entriesUser;
    }

    @Override
    public List<Cart> getValues() {
        List<Cart> entriesCart = new ArrayList<>();
        for (int i = 0; i < BUCKET_SIZE; i++) {
            for (Entry e : entries[i]) {
                entriesCart.add(e.getValue());
            }
        }
        return entriesCart;
    }

    @Override
    public Cart remove(User key) {
        int bucket = bucketIndex(key);
        Cart removedCart = null;
        for (int i=0; i<entries[bucket].size(); i++){
            if (entries[bucket].get(i).getKey() == key){
                removedCart = entries[bucket].get(i).getValue();
                entries[bucket].remove(entries[bucket].get(i));
            }
        }
        return removedCart;
    }

    private int bucketIndex(User key){
        return Math.abs(key.hashCode()) % entries.length;
    }
}
